// File memory.h for CC1B
#ifndef _MEMORY_H_
#define _MEMORY_H_
//include other libraries here
#pragma library 1

shadowDef char _SystemRam[256] @0x00;

#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_

/**
 * Calculate physical ram address from logical _SystemRam array index (0-143)
 * logical -> physical
 * 0x00-0x0F -> 0x00-0x0F
 * 0x10-0x1F -> 0x10-0x1F
 * 0x20-0x2F -> 0x30-0x3F
 * 0x30-0x3F -> 0x50-0x5F
 * 0x40-0x4F -> 0x70-0x7F
 * 0x50-0x5F -> 0x90-0x9F
 * 0x60-0x6F -> 0xB0-0xBF
 * 0x70-0x7F -> 0xD0-0xDF
 * 0x80-0x8F -> 0xF0-0xFF
 *
 * @param index Logical index into _SystemRam array
 * @return physical ram address
 */
char _SystemRamAddress(char index) {
  if ((W & 0xF0) != 0) {
    index += (index & 0xF0);
    index -= 0x10;
  }
  return index;
}

/**
 * Calculate logical _SystemRam array index (0-143) from physical ram address
 * physical -> logical
 * 0x00-0x0F -> 0x00-0x0F
 * 0x10-0x1F -> 0x10-0x1F
 * 0x30-0x3F -> 0x20-0x2F
 * 0x50-0x5F -> 0x30-0x3F
 * 0x70-0x7F -> 0x40-0x4F
 * 0x90-0x9F -> 0x50-0x5F
 * 0xB0-0xBF -> 0x60-0x6F
 * 0xD0-0xDF -> 0x70-0x7F
 * 0xF0-0xFF -> 0x80-0x8F
 *
 * @param addr Physical ram address
 * @return logical index into _SystemRam array
 */
char _SystemRamIndex(char addr) {
  if ((W & 0xF0) != 0) {
    addr -= 0x10;
    addr += (addr & 0x0F);
    addr = (addr >> 1) + 0x10;
  }
  return addr;
}

/**
 * Calculate physical ram address from array index
 *
 * @param base Physical ram address of array element 0
 * @param index Logical index into array
 * @return Physical ram address of array element index
 */
char LargeArrayAddress(char base, char index) {
  return _SystemRamAddress(_SystemRamIndex(base)+index);
}

/**
 * Calculate array index from physical ram address
 *
 * @param base Physical ram address of array element 0
 * @param addr Physical ram address of array element index
 * @return Logical index into array
 */
char LargeArrayIndex(char base, char addr) {
  addr = _SystemRamIndex(addr);
  return addr - _SystemRamIndex(base);
}

#endif

#if defined _CHIP_SX48_ || defined _CHIP_SX52_

/**
 * Calculate physical ram address from logical _SystemRam array index (0-255)
 * logical -> physical
 * 0x00-0xFF -> 0x00-0xFF
 *
 * @param index Logical index into _SystemRam array
 * @return physical ram address
 */
char _SystemRamAddress(char index) {
  return index;
}

/**
 * Calculate logical _SystemRam array index (0-255) from physical ram address
 * physical -> logical
 * 0x00-0xFF -> 0x00-0xFF
 *
 * @param addr Physical ram address
 * @return logical index into _SystemRam array
 */
char _SystemRamIndex(char addr) {
  return addr;
}

/**
 * Calculate physical ram address from array index
 *
 * @param base Physical ram address of array element 0
 * @param index Logical index into array
 * @return Physical ram address of array element index
 */
char LargeArrayAddress(char base, char index) {
  return base + index;
}

/**
 * Calculate array index from physical ram address
 *
 * @param base Physical ram address of array element 0
 * @param addr Physical ram address of array element index
 * @return Logical index into array
 */
char LargeArrayIndex(char base, char addr) {
  return addr - base;
}

#endif

/**
 * Read word from codepage (rom)
 *
 * @param addr Physical rom address
 *                        SX18/20/28: 0x000-0x7FF
 *                        SX48/52: 0x000-0xFFF
 * @return Word (12bits) at addr
 */
long RomWord(long addr) {
  long temp;
  MODE = addr.high8;
  W = addr.low8;
  iread();
  temp.low8 = W;
  temp.high8 = MODE;
  return temp;
}

/**
 * Read char from codepage (rom)
 *
 * @param addr Physical rom address
 *                        SX18/20/28: 0x000-0x7FF
 *                        SX48/52: 0x000-0xFFF
 * @return Char (8bits) at addr
 */
char RomChar(long addr) {
  MODE = addr.high8;
  W = addr.low8;
  iread();
  return W;
}

#pragma library 0
#endif
// End of file memory.h

